#from phabricator import Phabricator
import os, sys
import json
import numpy as np
import pandas as pd
import requests
import re
import datetime
import time


# the query task taken from MGerlach's notebook
def query_task_tag(
    tag_term, 
    limit = 100, 
    ts1 = None, ts2 = None,
    api_url_base = 'https://phabricator.wikimedia.org/api/maniphest.search',
    api_token = "api-wurg254ciq5uvfxlr4rszn5ynpy4",
    sleep = 10
):
    '''
    query all tasks tagged with specific tag
    OPTIONAL:
    - limit (int, default=100), number of results per query, cannot be larger than 100
    - ts1, ts2 (int, default = None); timewindow for creation of tasks (timestamp)
    - sleep (int, default = 0), sleep between each query
    - api_url_base (str,) is the url for the api
    - api_token (str, default=martins token), 

    RETURNS:
    - list of dictionary.
    '''
    time.sleep(sleep)
    to_query = 1
    after = None

    data = []

    while to_query == 1:
        time.sleep(sleep)
        params = {
            'api.token'  : api_token,
            'constraints[query]':[tag_term], ## term that task is searched for with
            #'constraints[projects]':["VisualEditor"], ## term that task is tagged with
            'constraints[createdStart]':ts1, ## timestamp task creation (min)
            'constraints[createdEnd]':ts2, ## timestamp task creation (max)
            'limit':limit,
            'after':after,
            "attachments[subscribers]":"true",
        }

        response = requests.get( api_url_base, params=params)
        print(response)
        result = json.loads(response.text)['result']
        print(result)
        ## the data
        data_tmp = result['data']
        data += data_tmp
        ## check if there are more results to query
        cursor = result['cursor']
        ## if after == None, no more queries
        if cursor['after'] == None:
            to_query = 0
        ## if after != None, query next page by passing after-argument
        else:
            after = cursor['after']
    return data



#also from MGerlach
def query_transactions_phid_task(
    task_phid, 
    limit = 100, 
    api_url_base = 'https://phabricator.wikimedia.org/api/transaction.search',
    api_token = 'api-grocgdq2767cx6v5ywckkjmetx2f',
    sleep = 10, 
):
    '''
    query all transactions for a task (task_phid).
    OPTIONAL:
    - limit (int, default=100), number of results per query, cannot be larger than 100
    - sleep (int, default = 0), sleep between each query
    - api_url_base (str,) is the url for the api
    - api_token (str, default=martins token), 

    RETURNS:
    - list of dictionary.
    '''
    time.sleep(sleep)
    to_query = 1
    after = None

    data = []

    while to_query == 1:
        time.sleep(sleep)
        params = {
            'api.token'  : api_token,
            'objectIdentifier':task_phid, ## task-phid
            'limit':limit,
            'after':after,
        }
        response = requests.get( api_url_base, params=params)
        result = json.loads(response.text)['result']

        ## the data
        data_tmp = result['data']
        data += data_tmp
        ## check if there are more results to query
        cursor = result['cursor']
        ## if after == None, no more queries
        if cursor['after'] == None:
            to_query = 0
        ## if after != None, query next page by passing after-argument
        else:
            after = cursor['after']
    return data

if __name__ == "__main__":
    # phab=Phabricator("https://phabricator.wikimedia.org/")
    tags = [
        "VisualEditor",
        "Parsoid"
    ]
    #set phabricator api token
    token = "api-wurg254ciq5uvfxlr4rszn5ynpy4"
    api_base = 'https://phabricator.wikimedia.org/api/'

    ve_ts1 = int(datetime.datetime.timestamp(datetime.datetime(2012, 1, 1, 0, 0, 0)))
    ve_ts2 = int(datetime.datetime.timestamp(datetime.datetime(2014, 12, 31, 0, 0, 0)))

    ve_data = query_task_tag("VisualEditor", ts1=ve_ts1, ts2=ve_ts2)

    for entry in ve_data:
        task_id = entry['phid']
        print(task_id)
        transactions = query_transactions_phid_task(task_id)
        comments = {}
        for item in transactions:
            comments[item['id']] = item['comments']
        entry['task_comments'] = comments
    DATA_PREFIX = "/data/users/mgaughan/mw-repo-lifecycles/phab_data/"
    with open(DATA_PREFIX + "visualeditor/" + "2012_1_1_to_2014_12_31.json", "w") as outfile1:
        json.dump(ve_data, outfile1)
